@@ -22,9 +22,8 @@
 
 from __future__ import with_statement
 
-VERSION = "0.8.0.4"
+VERSION = "0.8.0.5"
    
-
 import logging
 logging.basicConfig(level=logging.WARNING,
                     format='%(relativeCreated)6d %(levelname)10s %(name)-20s %(message)s',
@@ -50,6 +49,8 @@ import fieldnotesuploader
 from actors.tts import TTS
 #from actors.notify import Notify
 
+import connection
+
 if len(argv) == 1:
     import cli
     print cli.usage % ({'name': argv[0]})
@@ -143,6 +144,7 @@ class Core(gobject.GObject):
         'options_rotate_screen': 0,
         'tts_interval': 0,
         'options_default_log_text' : 'TFTC!\n\nLogged at %X from my %(machine)s using AGTL.',
+        'options_auto_update': True,
     }
             
     def __init__(self, guitype, root):
@@ -160,14 +162,15 @@ class Core(gobject.GObject):
         self.connect('save-settings', self.__on_save_settings)
         self.create_recursive(self.settings['download_output_dir'])
         self.create_recursive(self.settings['download_map_path'])
-                
+     
+        connection.init()
 
         self.downloader = downloader.FileDownloader(self.settings['options_username'], self.settings['options_password'], self.COOKIE_FILE, cachedownloader.GeocachingComCacheDownloader.login_callback)
                 
         self.pointprovider = provider.PointProvider(self.CACHES_DB, geocaching.GeocacheCoordinate, 'geocaches')
 
         self.gui = guitype(self, self.dataroot)
-
+        
         
         actor_tts = TTS(self)
         actor_tts.connect('error', lambda caller, msg: self.emit('error', msg))
@@ -255,7 +258,7 @@ class Core(gobject.GObject):
             return "%d MiB" % (size / (1024 * 1024))
         else:
             return "%d GiB" % (size / (1024 * 1024 * 1024))
-
+            
 
     ##############################################
     #
@@ -281,12 +284,18 @@ class Core(gobject.GObject):
                         reload(m)
                         updated_modules += 1
                     else:
-                        logging.info("Not reloading Module '%s', current version number: %d, version number of update file: %d" % (m.__name__, v_dict['VERSION'], m.VERSION))
+                        logging.info("Not reloading Module '%s', current version number: %d, version number of update file: %d" % (m.__name__, m.VERSION, v_dict['VERSION']))
                 else:
                     logging.info("Skipping nonexistant update from" + path.join(self.UPDATE_DIR, "%s%spy" % (m.__name__, extsep)))
         return updated_modules
 
-    def try_update(self):
+    def try_update(self, silent = False):
+        if connection.offline:
+            if not silent:
+                self.emit('error', Exception("Can't update in offline mode."))
+            return False
+            
+            
         from urllib import urlretrieve
         from urllib2 import HTTPError
         import tempfile
@@ -344,10 +353,11 @@ class Core(gobject.GObject):
                     except Exception:
                         pass
         except Exception, e:
-            logging.exception(e)
             def same_thread(error):
+                logging.exception(e)
                 self.emit('hide-progress')
-                self.emit('error', error)
+                if not silent:
+                    self.emit('error', error)
                 return False
             gobject.idle_add(same_thread, e)
             return None
@@ -655,7 +665,7 @@ class Core(gobject.GObject):
 
     # called on signal by downloading thread
     def on_download_error(self, something, error):
-        extra_message = "Please try the online update in the about menu. Error:\n%s" % error
+        extra_message = "Error:\n%s" % error
         logging.exception(error)
         def same_thread(error):
             self.emit('hide-progress')
