@@ -1,8 +1,8 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#        Copyright (C) 2009 Daniel Fett
-#         This program is free software: you can redistribute it and/or modify
+#   Copyright (C) 2010 Daniel Fett
+#   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
 #   (at your option) any later version.
@@ -15,12 +15,15 @@
 #   You should have received a copy of the GNU General Public License
 #   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-#        Author: Daniel Fett advancedcaching@fragcom.de
+#   Author: Daniel Fett agtl@danielfett.de
+#   Jabber: fett.daniel@jaber.ccc.de
+#   Bugtracker and GIT Repository: http://github.com/webhamster/advancedcaching
 #
 
 
 import logging
 logger = logging.getLogger('downloader')
+import connection
 
 class FileDownloader():
     USER_AGENT = 'User-Agent: Mozilla/5.0 (X11; U; Linux i686; de; rv:1.9.0.12) Gecko/2009070811  Windows NT Firefox/3.1'
@@ -50,6 +53,8 @@ class FileDownloader():
 
 
     def login(self):
+        if connection.offline:
+            raise Exception("Can't connect in offline mode.")
         if self.username == '' or self.password == '':
             raise Exception("Please configure your username/password and restart the application")
         logger.info("Checking Login status")
@@ -74,9 +79,11 @@ class FileDownloader():
                 if 'You are logged in as' in line:
                     self.logged_in = True
                     logger.info("Seems as we're still logged in")
+                    page.close()
                     return
                 elif 'You are not logged in.' in line:
                     logger.info("Nope, not logged in anymore")
+                    page.close()
                     break
         
         logger.info("Logging in")
@@ -102,6 +109,8 @@ class FileDownloader():
 
 
     def get_reader(self, url, values=None, data=None, login = True):
+        if connection.offline:
+            raise Exception("Can't connect in offline mode.")
         from urllib import urlencode
         from urllib2 import Request, urlopen
         if login and not self.logged_in:
