@@ -1,8 +1,8 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#    Copyright (C) 2009 Daniel Fett
-#     This program is free software: you can redistribute it and/or modify
+#   Copyright (C) 2010 Daniel Fett
+#   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
 #   (at your option) any later version.
@@ -15,7 +15,9 @@
 #   You should have received a copy of the GNU General Public License
 #   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-#    Author: Daniel Fett advancedcaching@fragcom.de
+#   Author: Daniel Fett agtl@danielfett.de
+#   Jabber: fett.daniel@jaber.ccc.de
+#   Bugtracker and GIT Repository: http://github.com/webhamster/advancedcaching
 #
 
 import math
@@ -31,7 +33,7 @@ except Exception:
         dlon = math.pow(math.sin(math.radians(target.lon-src.lon) / 2), 2)
         a = dlat + math.cos(math.radians(src.lat)) * math.cos(math.radians(target.lat)) * dlon;
         c = 2 * math.atan2(math.sqrt(a), math.sqrt(1-a));
-        return self.RADIUS_EARTH * c;
+        return Coordinate.RADIUS_EARTH * c;
     distance_to = distance_to_manual
 
 def try_parse_coordinate(text):
@@ -138,7 +140,7 @@ class Coordinate(object):
     re_to_dm_array = re.compile('^(\d?)(\d)(\d) (\d)(\d)\.(\d)(\d)(\d)$')
     re_to_d_array = re.compile('^(\d?)(\d)(\d).(\d)(\d)(\d)(\d)(\d)$')
     
-    def __init__(self, lat, lon, name="No Name"):
+    def __init__(self, lat, lon, name=""):
         self.lat = lat
         self.lon = lon
         self.name = name
@@ -258,4 +260,35 @@ class Coordinate(object):
         self.name = data['name']
         
     def distance_to(self, target):
-        return distance_to(self, target)
\ No newline at end of file
+        return distance_to(self, target)
+
+    @staticmethod
+    def format_distance(distance):
+        if distance == None:
+            return '?'
+        if distance >= 1000:
+            return "%d km" % round(distance / 1000.0)
+        elif distance >= 100:
+            return "%d m" % round(distance)
+        else:
+            return "%.1f m" % round(distance, 1)
+
+    @staticmethod
+    def format_direction(angle):
+        directions = ['N', 'NNE', 'NE', 'ENE', 'E', 'ESE', 'SE', 'SSE',
+                        'S', 'SSW', 'SW', 'WSW', 'W', 'WNW', 'NW', 'NNW', 'N']
+
+        return directions[int(round(((angle+360.0) % 360.0)/22.5))]
+ 
+    @staticmethod
+    def get_bounds(coord_list):
+        min_lat = coord_list[0].lat
+        max_lat = coord_list[0].lat
+        min_lon = coord_list[0].lon
+        max_lon = coord_list[0].lon
+        for x in coord_list:
+            min_lat = min(min_lat, x.lat)
+            max_lat = max(max_lat, x.lat)
+            min_lon = min(min_lon, x.lon)
+            max_lon = max(max_lon, x.lon)
+        return min_lat, max_lat, min_lon, max_lon
